{% if data and data.object_id %}
  {% if "eln_source_url" in data and data.eln_source_url %}
    {% if "eln_object_url" in data and data.eln_object_url %}
      <a href="{{ data.eln_object_url }}">
    {% endif %}
    {{ _('Object') }} #{{ data.object_id }} @
    {% if not ("eln_object_url" in data and data.eln_object_url) %}
      <a href="{{ data.eln_source_url }}">
    {% endif %}
    {{ data.eln_source_url }}
    </a>
  {% else %}
  {% if "component_uuid" in data and data.component_uuid is not none and data.component_uuid != config['FEDERATION_UUID'] %}
      {% set component_uuid = data.component_uuid %}
    {% else %}
      {% set component_uuid = none %}
    {% endif %}
    {% with obj = get_object_if_current_user_has_read_permissions(data.object_id, component_uuid) %}
      {% if not (obj is none) %}
        <a href="{{ url_for('frontend.object', object_id=obj.object_id, _external=True) }}">{{ obj.name | get_translated_text(metadata_language) or '—' }} (#{{ obj.object_id }})</a>
        {{ import_information_symbol(obj, text=true) }}
      {% elif component_uuid is not none %}
        {{ _('Object') }} #{{ data.object_id }}
        {{ import_information_symbol(FederationObjectRef(fed_id=data.object_id, component_uuid=component_uuid), text=true) }}
      {% else %}
        <a href="{{ url_for('frontend.object', object_id=data.object_id, _external=True) }}">#{{ data.object_id }}</a>
      {% endif %}
    {% endwith %}
  {% endif %}
{% else %}
  &mdash;
{% endif %}
