<div class="form-group error-parent" style="padding-right:0.75em; display: flex; margin: 0 5px;" data-name="{{ id_prefix }}_parent">
  {% block property_label %}
    {% set property_label_width = "auto" %}
    {% set property_label_class = "" %}
    {% include "objects/forms/property_label.html" %}
  {% endblock %}
  <div style="padding-left: 5px">
    {% block property_field %}
    {% endblock %}
    {% block property_errors %}
    <span class="help-block error-note"></span>
    {% endblock %}
    {% if 'note' in schema %}
      <span class="help-block"><strong>{{ _('Note:') }}</strong> {{ schema['note'] | get_translated_text }}</span>
    {% endif %}
  </div>
</div>
