{% extends "objects/inline_edit/regular_property.html" %}

{% block property_value %}
  {% include "objects/values/hazards_value.html" %}
{% endblock %}

{% block property_field %}
  {% include "objects/fields/hazards_field.html" %}
{% endblock %}
