<textarea
  class="form-control text-multiline-field"
  {% if num_rows is integer %}
    rows="{{ num_rows }}"
  {% endif %}
  name="{{ id_prefix }}_text"
  placeholder="{{ (schema.placeholder or schema.title) | get_translated_text }}"
>
  {%- if id_prefix+'_text' in form_data -%}
    {{ form_data[id_prefix+'_text'] }}
  {%- elif data is not none and "text" in data -%}
    {{ data.text | get_all_translated_texts('\n') }}
  {%- endif -%}
</textarea>
