{% extends get_inline_edit_property_template(schema, container_style) %}

{% block property_value %}
  {% include "objects/values/object_reference_value.html" %}
{% endblock %}

{% block property_field %}
  {% include 'objects/fields/object_reference_base_field.html' %}
{% endblock %}

{% block property_other %}
  {% include "objects/view/include_object_reference_base.html" %}
{% endblock %}
