{% extends "base.html" %}

{% block title %}{{ _('Export to %(dataverse_name)s', dataverse_name=config['DATAVERSE_NAME']) }}— {{ service_name }}{% endblock %}

{% block content %}
  <h1>{{ _('Export to Dataverse') }}</h1>
  <p>{{ _('This object has already been exported as:') }} <a href="{{ dataverse_url }}">{{ dataverse_url }}</a>.</p>
  <p><a href="{{ url_for('.object', object_id=object_id) }}" class="btn btn-default">{{ _('Back') }}</a></p>
{% endblock %}
