<textarea type="text" class="form-control plotly-chart-editor" name="{{ id_prefix }}_plotly" rows="5" placeholder="">{% if id_prefix+'_plotly' in form_data %}{{ form_data[id_prefix+'_plotly'] }}{% elif data is not none and "plotly" in data %}{{ data.plotly | to_json_no_extra_escapes(indent=None) }}{% endif %}</textarea>
