{% if data and data.text %}
{% if data.is_markdown %}
  <div class="object-user-content">{{ data.text | get_translated_text(metadata_language) | markdown_to_safe_html | safe }}</div>
{% elif schema.multiline %}
  <div class="object-user-content multiline">{{ data.text | get_translated_text(metadata_language) }}</div>
{% else %}
  {{ data.text | get_translated_text(metadata_language) }}
{% endif %}
{% else %}
  &mdash;
{% endif %}
