{% set container_style = none %}
<div class="row" style="padding-right: 0.75em;">
  {% include "objects/property_label.html" %}
</div>
<div style="padding-left:2.5em; padding-right: 15px; overflow-wrap: anywhere">
  {% if data is not none %}
    {% for item in data %}
      {% set schema = schema['items'] %}
      {% set property_path = property_path + (loop.index0,) %}
      {% set id_prefix = id_prefix_for_property_path(property_path, id_prefix_root) %}
      {% set data = item %}
      {% if diff is not none and previous_schema is not none and not (diff is mapping) and diff | length > loop.index0  %}
        {% set diff = diff[loop.index0] %}
        {% if 'items' in previous_schema %}
          {% set previous_schema = previous_schema['items'] %}
        {% else %}
          {% set previous_schema = none %}
        {% endif %}
      {% else %}
        {% set diff = none %}
        {% set previous_schema = none %}
      {% endif %}
      <div>
        <div style="border:1px solid #cccccc; margin-right:-1px; border-radius:5px 0 0 5px; padding-left:2.5em; padding-top:1em; margin-bottom: 1em;  padding-bottom: 0.5em;">
          {% set show_indent_border = False %}
          {% include "objects/view/any.html" %}
        </div>
      </div>
    {% endfor %}
  {% endif %}
</div>
