{% set container_style = schema.get('style') %}
<div class="row" style="padding-right: 0.75em">
  {% include "objects/property_label.html" %}
  {% set is_required = True %}
  <input type="hidden" name="{{ id_prefix }}_hidden" value="array" />
  <div class="col-md-9 array-style-list">
    {% if data is not none %}
      <ul style="padding-left: 1.25em">
        {% for item in data %}
          {% set schema = schema['items'] %}
          {% set property_path = property_path + (loop.index0,) %}
          {% set id_prefix = id_prefix_for_property_path(property_path, id_prefix_root) %}
          {% set data = item %}
          <li>
            {% include "objects/inline_edit/inline_edit_any.html" %}
          </li>
        {% endfor %}
      </ul>
    {% endif %}
  </div>
</div>
