{% extends "objects/forms/list_property.html" %}

{% block property_field %}
  {% if schema.markdown %}
   {% include "objects/fields/text_markdown_field.html" %}
  {% elif schema.multiline  %}
   {% set num_rows = none %}
   {% include "objects/fields/text_multiline_field.html" %}
  {% elif 'choices' in schema %}
   {% include "objects/fields/text_choices_field.html" %}
  {% else %}
    {% include "objects/fields/text_plain_field.html" %}
  {% endif %}
{% endblock %}
