{% set container_style = schema.get('style') %}
<div class="row" style="padding-right: 0.75em;">
  {% include "objects/property_label.html" %}
  <div class="col-md-9 array-style-list">
  {% if data %}
    <ul style="padding-left: 1.25em">
      {% for item in data %}
        {% set schema = schema['items'] %}
        {% set property_path = property_path + (loop.index0,) %}
        {% set id_prefix = id_prefix_for_property_path(property_path, id_prefix_root) %}
        {% set data = item %}
        {% if diff is not none and previous_schema is not none and not (diff is mapping) and diff | length > loop.index0  %}
          {% set diff = diff[loop.index0] %}
          {% if 'items' in previous_schema %}
            {% set previous_schema = previous_schema['items'] %}
          {% else %}
            {% set previous_schema = none %}
          {% endif %}
        {% else %}
          {% set diff = none %}
          {% set previous_schema = none %}
        {% endif %}
        <li>
          {% include "objects/view/any.html" %}
        </li>
      {% endfor %}
    </ul>
    {% if data.export_edit_note %}
      <p class="text-muted"><i class="fa fa-share-alt" aria-hidden="true"></i> _('Note:') {{ data.export_edit_note }}</p>
    {% endif %}
  {% else %}
    &mdash;
  {% endif %}
  </div>
</div>
