{% extends "objects/inline_edit/inline_edit_text.html" %}

{% block property_label %}
  {% set property_label_width = "auto" %}
  {% set property_label_class = "" %}
  {% include "objects/property_label.html" %}
{% endblock %}

{% block property_field %}
  {% include "objects/forms/form_text_language_picker.html" %}
  {% include "objects/fields/text_field.html" %}
{% endblock %}
