{% extends "objects/inline_edit/table_property.html" %}

{% block property_value %}
  {% include "objects/values/text_value.html" %}
{% endblock %}

{% block property_edit_helper %}
  {% if data and data.text | get_translated_text(metadata_language) and ((schema.markdown and data.is_markdown) or schema.multiline) %}
    <a class="edit-helper" style="position: absolute; bottom: 10px; right: 13px;"><i class="fa fa-pencil-square-o"></i></a>
  {% else %}
    <a class="edit-helper"><i class="fa fa-pencil-square-o"></i></a>
  {% endif %}
{% endblock %}

{% block property_field %}
  {% if schema.multiline or schema.markdown %}
    {% set num_rows = 1 %}
    {% include "objects/fields/text_multiline_field.html" %}
  {% elif 'choices' in schema %}
    {% include "objects/fields/text_choices_field.html" %}
  {% else %}
    {% include "objects/fields/text_plain_field.html" %}
  {% endif %}
{% endblock %}
