<ul class="object-property-list">
{% if schema is none or data is none %}
  <li>{{ _('This object has been imported without data.') }}</li>
{% else %}
  {% set property_names = get_property_names_in_order(schema) %}
  {% for property_name in property_names %}
    {% if data | are_conditions_fulfilled(schema.properties[property_name]) %}
      {% set schema = schema['properties'][property_name] %}
      {% set data = data.get(property_name) %}
      <li>{{ schema.title | get_translated_text(metadata_language) }}: {% include "pdfexport/any.html" %}</li>
    {% endif %}
  {% endfor %}
{% endif %}
</ul>
