{% extends get_form_property_template(schema, container_style) %}

{% block property_field %}
  {% include "objects/fields/object_reference_base_field.html" %}
{% endblock %}

{% block property_errors %}
  {% if config['USE_TYPEAHEAD_FOR_OBJECTS'] %}
    <span class="help-block"></span>
  {% endif %}
  {{ super() }}
{% endblock %}
