<div class="error-parent form-group row form-list-property" style="padding-right:0.75em; flex-grow: 1">
  {% block property_field %}
  {% endblock %}
  <span class="help-block error-note"></span>
</div>
