{% if instrument is not none and instrument.responsible_users %}
  {% set instrument_scientists = [] %}
  {% for user in instrument.responsible_users %}
    {% if not user.is_hidden or current_user.is_admin %}
      {{ instrument_scientists.append(user) or '' }}
    {% endif %}
  {% endfor %}
  {% if instrument_scientists %}
<div class="instrument-scientists">
  <h3>{{ _("Instrument Scientists:") }}</h3>
  <ul>
  {% for user in instrument_scientists %}
    <li>
      {{ user_reference(user) }}
      {% if user.is_hidden %}
      <span class="label label-default" data-toggle="tooltip" data-placement="top" title="{{ _('Accounts marked as hidden will not be shown in user lists.') }}">{{ _('hidden') }}</span>
      {% endif %}
    </li>
  {% endfor %}
  </ul>
</div>
  {% endif %}
{% endif %}
