{% if property_label_class is not defined %}
  {% set property_label_class = 'col-md-3' %}
{% endif %}
<label class="control-label {{ property_label_class }} {% if schema.title | get_translated_text and is_required %}required-label{% endif %}">
{% if 'tooltip' in schema %}
  <span {% if is_template %} data-template-toggle="tooltip" {% else %} data-toggle="tooltip" {% endif %} data-placement="top" title="{{ schema.tooltip | get_translated_text }}">
{% endif %}
{{ schema.title | get_translated_text }}
{% if 'tooltip' in schema %}
    <i class="fa fa-question-circle" aria-hidden="true"></i>
  </span>
{% endif %}
</label>
