{% set container_style = schema.get('style') %}
<div class="row" style="padding-right: 0.75em;">
  {% include "objects/property_label.html" %}
  {% set is_required = False %}
</div>
<div style="padding-left:2.5em;">
  <input type="hidden" name="{{ id_prefix }}_hidden" value="array" />
  {% if data is not none %}
    {% for item in data %}
      <input type="hidden" name="{{ id_prefix }}_{{ loop.index0 }}__hidden" value="{{ schema["items"]["type"] }}" />
    {% endfor %}
  {% endif %}
  {% if schema["items"]["type"] == 'object' %}
    <table class="table">
      <tbody>
        {% for property_name in get_property_names_in_order(schema["items"]) %}
          <tr>
            {% with %}
              {% set schema = schema["items"].properties[property_name] %}
              {% set property_path = property_path + ('?', property_name) %}
              <th scope="col">
                {% include "objects/table_property_label.html" %}
              </th>
            {% endwith %}
            {% if data is not none %}
              {% for item in data %}
                <td>
                  {% set is_required = property_name in schema['items'].get('required', ()) %}
                  {% set schema = schema['items'].properties[property_name] %}
                  {% if property_name in item %}
                    {% set data = item[property_name] %}
                  {% else %}
                    {% set data = none %}
                  {% endif %}
                  {% set entry_index = loop.index0 %}
                  {% set property_path = property_path + (loop.index0, property_name,) %}
                  {% set id_prefix = id_prefix_for_property_path(property_path, id_prefix_root) %}
                  {% if 'conditions' in schema %}
                    <div data-condition-wrapper-for="{{ id_prefix }}" class="condition-wrapper">
                      {% include "objects/inline_edit/inline_edit_any.html" %}
                      <input type="hidden" class="condition-wrapper" data-id-prefix="{{ id_prefix}}" data-conditions='{{ schema["conditions"] | encode_choices | stringify }}'>
                    </div>
                    <div data-condition-replacement-for="{{ id_prefix }}" class="condition-replacement">
                      &mdash;
                    </div>
                  {% else %}
                    {% include "objects/inline_edit/inline_edit_any.html" %}
                  {% endif %}
                </td>
              {% endfor %}
            {% endif %}
          </tr>
        {% endfor %}
      </tbody>
    </table>
  {% else %}
    <p>{{ _('Unsupported item type for horizontal_table style.') }}</p>
  {% endif %}
</div>
