<div class="row form-area inline-edit-regular-property" data-name="{{ id_prefix }}_parent">
  {# Label #}
  {% block property_label %}
    {% include "objects/property_label.html" %}
  {% endblock %}
  {# View #}
  <div class="view-switch col-md-9 width-md-important" style="padding-left: 15px; width: calc(75% + {{ (indent_level-1) * (10 * 0.75 + 0.75 * 5) }}px);">
    <div class="alert-upload-container">
      {% block property_value %}
      {% endblock %}
      {% block property_edit_helper %}
        <a class="edit-helper"><i class="fa fa-pencil-square-o"></i></a>
      {% endblock %}
      <div class="alert-upload-failed">
        {{ _('Your input is not valid. Please check.') }}
      </div>
    </div>
  </div>
  {# Form #}
  <div class="form-switch">
    <div class="col-md-9 width-md-important" style="width: calc(75% + {{ (indent_level-1) * (10 * 0.75 + 0.75 * 5) }}px);">
      {% block property_field %}
      {% endblock %}
      {% if 'note' in schema %}
        <span class="help-block"><strong>{{ _('Note:') }}</strong> {{ schema['note'] | get_translated_text }}</span>
      {% endif %}
    </div>
  </div>
</div>

{% block property_other %}
{% endblock %}
