<div class="row" style="padding-right:0.75em">
  {% block property_label %}
    {% include "objects/property_label.html" %}
  {% endblock %}
  <div class="col-md-9 width-md-important" style="padding-left: 15px; width: calc(75% + {{ (indent_level-1) * (10 * 0.75 + 0.75 * 5) }}px)">
    {% block property_field %}
    {% endblock %}
    {% if data and data.export_edit_note %}
      <p class="text-muted"><i class="fa fa-share-alt" aria-hidden="true"></i> {{ _('Note:') }} {{ data.export_edit_note }}</p>
    {% endif %}
  </div>
</div>
{% block property_other %}
{% endblock %}
