<div class="form-group row" style="padding-right: 0.75em">
  <div data-id-prefix="{{ id_prefix }}" style="padding-right: 15px" class="pull-right div-apply-recipe" {% if is_template %} data-template-toggle="tooltip" {% else %} data-toggle="tooltip" {% endif %} data-placement="left" title="" data-sampledb-recipes="{{ schema.recipes | stringify }}">
    <select class="form-control">
      {% for recipe in schema.recipes %}
        <option value="{{ loop.index0 }}">{{ recipe.name }}</option>
      {% endfor %}
    </select>
    <button type="button" class="btn btn-primary input-group-button-for-selectpicker"
            aria-label="{{ _('Apply Recipe') }}" name="apply" value="Apply Recipe">
      <span aria-hidden="true">{{ _('Apply Recipe') }}</span>
    </button>
  </div>
</div>