{% extends "errors/base.html" %}

{% block title %}{{ _('Forbidden') }} — {{ service_name }}{% endblock %}

{% block content %}
<h1>403 &mdash; {{ _('Forbidden') }}</h1>
<p>{{ _('You are not authorized to view the page you tried to open.') }}</p>
{% endblock %}
