{% set container_style = none %}
<div class="form-group row">
  {% if schema.title %}
    {% include "objects/property_label.html" %}
  {% endif %}
  {% set is_required = True %}
</div>
<div style="padding-left:2.5em; padding-right: 15px; overflow-wrap: anywhere">
  <input type="hidden" name="{{ id_prefix }}_hidden" value="array" />
  {% if data is not none %}
    {% for item in data %}
      {% set schema = schema['items'] %}
      {% set property_path = property_path + (loop.index0,) %}
      {% set id_prefix = id_prefix_for_property_path(property_path, id_prefix_root) %}
      {% set data = item %}
      <div style="border-left:1px solid #cccccc; border-bottom:1px solid #cccccc; border-top:1px solid #cccccc; border-right:1px solid #cccccc; margin-right:-1px; border-radius:5px 0 0 5px; padding-left:2.5em; padding-top:0.5em; padding-bottom: 0.5em; margin-bottom: 4px;">
        {% set show_indent_border = False %}
        {% include "objects/inline_edit/inline_edit_any.html" %}
      </div>
    {% endfor %}
  {% endif %}
</div>
