{% extends "base.html" %}

{% block title %}{{ _('Warnings') }} — {{ service_name }}{% endblock %}

{% block content %}
  <h1>{{ _('Warnings') }}</h1>
  {% if has_warnings %}
    <ul>
    {% if show_numeric_tags_warning %}
        <li>
        {% if do_numeric_tags_exist %}
          {{ _('There are numeric tags, please evaluate if these are necessary for your use case and set the configuration value <code>ENABLE_NUMERIC_TAGS</code> to <code>False</code> to disable them.') }}
        {% else %}
          {{ _('Numeric tags are enabled, please do not set the configuration value <code>ENABLE_NUMERIC_TAGS</code> or set it to <code>False</code>.') }}
        {% endif %}
          <a href="https://scientific-it-systems.iffgit.fz-juelich.de/SampleDB/administrator_guide/deprecated_features.html#numeric-tags">{{ _('Read more.') }}</a>
        </li>
    {% endif %}
    </ul>
  {% else %}
    <p class="text-muted">{{ _('There are no warnings for this installation of SampleDB.') }}</p>
  {% endif %}
{% endblock %}
