{% set container_style = schema.get('style') %}
<div class="row" style="padding-right: 0.75em;">
  {% include "objects/property_label.html" %}
</div>
<div style="padding-left:2.5em;">
  {% if schema["items"]["type"] == 'object' %}
    <table class="table">
      <tbody>
        {% for property_name in get_property_names_in_order(schema["items"]) %}
          <tr>
            {% with %}
              {% set schema = schema["items"].properties[property_name] %}
              {% set property_path = property_path + ('?', property_name) %}
              <th scope="col">
                {% include "objects/table_property_label.html" %}
              </th>
            {% endwith %}
            {% if data is not none %}
              {% for item in data %}
                <td>
                  {% set schema = schema['items'].properties[property_name] %}
                  {% if property_name in item %}
                    {% set data = item[property_name] %}
                  {% else %}
                    {% set data = none %}
                  {% endif %}
                  {% set entry_index = loop.index0 %}
                  {% set property_path = property_path + (entry_index, property_name) %}
                  {% include "objects/view/any.html" %}
                </td>
              {% endfor %}
            {% endif %}
          </tr>
        {% endfor %}
      </tbody>
    </table>
  {% else %}
    <p>{{ _('Unsupported item type for horizontal_table style.') }}</p>
  {% endif %}
</div>
