{% extends "objects/forms/table_property.html" %}

{% block property_field %}
  {% if schema.multiline or schema.markdown %}
    {% set num_rows = 1 %}
    {% include "objects/fields/text_multiline_field.html" %}
  {% elif 'choices' in schema %}
    {% include "objects/fields/text_choices_field.html" %}
  {% else %}
    {% include "objects/fields/text_plain_field.html" %}
  {% endif %}
{% endblock %}
