{% extends "errors/base.html" %}

{% block title %}{{ _('%(dataverse_name)s not reachable', dataverse_name=config['DATAVERSE_NAME']) }} — {{ service_name }}{% endblock %}

{% block content %}
  <h1>{{ _('%(dataverse_name)s not reachable', dataverse_name=config['DATAVERSE_NAME']) }}</h1>
  <p>{{ _('%(service_name)s cannot reach the %(dataverse_name)s API at this time. Please try again later.', service_name=service_name, dataverse_name=config['DATAVERSE_NAME']) }}</p>
{% endblock %}
