.row.alert {
    padding-left: 0px;
}

.form-switch {
    display: none;
}

.view-switch {
    display: block;
}

.alert-upload-failed {
    display: none
}

.alert-upload-container {
    padding-left: 0px;
}

.form-area {
    margin-bottom: 0px;
}

.title-label {
    margin-top: 5px;
}

.input-list {
    display: inline-block;
    vertical-align: middle;
    width: 100%;
}

.edit-helper {
    padding-left: 0.8em;
    visibility: hidden;
    cursor: pointer ;
}

.inline-edit-regular-property {
    padding-right:0.75em;
}

.inline-edit-table-property {
    position: relative;
}

.inline-edit-horizontal-property {
    padding-right:0.75em;
    display: flex;
}

.inline-edit-horizontal-property .form-switch {
    margin-bottom: 5px;
}

.inline-edit-table-property .form-switch .row {
    padding-right: 0.75em;
    padding-left: 1em;
}

.inline-edit-horizontal-property .form-switch .checkbox,
.inline-edit-regular-property .form-switch .checkbox,
.inline-edit-table-property .form-switch .checkbox {
    padding-top: 0;
    min-height: initial;
}

.inline-edit-horizontal-property .form-switch .checkbox label,
.inline-edit-regular-property .form-switch .checkbox label,
.inline-edit-table-property .form-switch .checkbox label {
    margin-bottom: 0;
    min-height: 15px;
}

.inline-edit-list-property textarea.text-multiline-field{
    min-height:2.5em;
    height:4em;
    margin-bottom: 14px;
}

.inline-edit-list-property textarea.text-markdown-field {
    min-height: 171px;
    margin-bottom: 14px;
}
