{% if property_label_class is not defined %}
  {% set property_label_class = 'col-md-3 width-md-important' %}
{% endif %}
<label class="{{ property_label_class }}" style="text-align: right; width: {% if property_label_width %}{{ property_label_width }}{% else %}calc(25% - {{ indent_level * 10 * 0.75 + 5 * 0.75 * (indent_level-1) }}px){% endif %};">
  {% if 'tooltip' in schema %}
    <span data-toggle="tooltip" data-placement="top" title="{{ schema.tooltip | get_translated_text }}">
      {{ schema.title | get_translated_text }}
      <i class="fa fa-question-circle" aria-hidden="true"></i>
    </span>
  {% else %}
    {{ schema.title | get_translated_text }}
  {% endif %}
  {% if not hide_search_button %}
    {% set search_property_path = (search_property_path_prefix or ()) + property_path %}
    {% set search_url = get_search_url(search_property_path, data, metadata_language=metadata_language) %}
    {% if search_url %}
      <a href="{{ search_url }}" class="search-helper"><i class="fa fa-search" aria-hidden="true"></i></a>
    {% else %}
      <i class="fa fa-search" style="visibility:hidden" aria-hidden="true"></i>
    {% endif %}
  {% endif %}
</label>
