{% if 'action_type_id' not in schema or schema['action_type_id'] is none %}
  {% set schema_action_type_ids = none %}
{% elif schema['action_type_id'] is integer %}
  {% set schema_action_type_ids = [schema['action_type_id']] %}
{% else %}
  {% set schema_action_type_ids = schema['action_type_id'] %}
{% endif %}
{% if 'action_id' not in schema or schema['action_id'] is none %}
  {% set schema_action_ids = none %}
{% elif schema['action_id'] is integer %}
  {% set schema_action_ids = [schema['action_id']] %}
{% else %}
  {% set schema_action_ids = schema['action_id'] %}
{% endif %}
{% set filter_operator = schema.get('filter_operator', 'and') %}
{% include 'objects/forms/form_object_reference_base.html' %}
